import 'dart:convert';
import 'dart:core';

import 'package:crypto/crypto.dart';

extension MapExtraUtils on Map {
  Map Concat(Map other) {
    other?.forEach((key, value) {
      if(!this.containsKey(key))
        this[key] = value;
    });

    return this;
  }

  Map ConcatToNew(Map other) {
    Map res = {};

    this.forEach((key, value) {
      res[key] = value;
    });

    other?.forEach((key, value) {
      res[key] = value;
    });

    return res;
  }

  String get Json {
    return jsonEncode(this);
  }

  String get JsonPretty {
    return JsonEncoder.withIndent('\t').convert(this);
  }

  String JWTToken(String secret) {
    var toSing = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzUxMiJ9.' + base64Url.encode(utf8.encode(jsonEncode(this)));

    return toSing + '.' + base64Url.encode(Hmac(sha512, utf8.encode(secret)).convert(utf8.encode(toSing)).bytes);
  }

  void ForEach(bool Function(dynamic key, dynamic value) step) {
    for(var key in this.keys) {
      if(step(key,this[key]) == false)
        break;
    }
  }

  void ForEacCounter(bool Function(dynamic key, dynamic value, int index) step) {
    int i = 0;
    for(var key in this.keys) {
      if(step(key,this[key],i) == false)
        break;
      i++;
    }
  }
}
